@@ -6,22 +6,24 @@ __all__ = [
     "get_dir_count", "has_id", "iter_existing_id", "get_parent_id", "iter_parent_id", 
     "iter_id_to_parent_id", "iter_id_to_path", "id_to_path", "get_id", "get_pickcode", 
     "get_sha1", "get_path", "get_ancestors", "get_attr", "iter_children", 
-    "iter_descendants", "iter_descendants_fast", "iter_files_with_path_url", 
-    "select_na_ids", "select_mtime_groups", "dump_to_alist", 
+    "iter_descendants", "iter_descendants_bfs", "iter_files_with_path_url", 
+    "iter_dup_files", "iter_dangling_parent_ids", "iter_dangling_ids", "select_na_ids", 
+    "select_mtime_groups", "dump_to_alist", "dump_efu", 
 ]
 
 from collections.abc import Callable, Iterable, Iterator, Sequence
+from csv import writer
 from datetime import datetime
 from errno import ENOENT, ENOTDIR
 from itertools import batched
+from ntpath import normpath
 from os.path import expanduser
 from pathlib import Path
-from sqlite3 import register_adapter, register_converter, Connection, Cursor, OperationalError
-from posixpath import join
-from typing import cast, overload, Any, Final, Literal
-from urllib.parse import quote
+from sqlite3 import register_converter, Connection, Cursor
+from typing import overload, Any, Final, Literal
 
-from iterutils import bfs_gen
+from iterutils import bfs_gen, group_collect
+from orjson import loads
 from posixpatht import escape, path_is_dir_form, splits
 from sqlitetools import find, query, transact
 
@@ -30,20 +32,24 @@ FIELDS: Final = (
     "id", "parent_id", "pickcode", "sha1", "name", "size", "is_dir", "type", 
     "ctime", "mtime", "is_collect", "is_alive", "updated_at", 
 )
-EXTENDED_FIELDS: Final = (*FIELDS, "path", "posixpath")
+EXTENDED_FIELDS: Final = (*FIELDS, "depth", "path", "posixpath", "ancestors")
 
 register_converter("DATETIME", lambda dt: datetime.fromisoformat(str(dt, "utf-8")))
+register_converter("JSON", loads)
 
 
 def get_dir_count(
     con: Connection | Cursor, 
-    id: int, 
+    id: int = 0, 
     /, 
+    is_alive: bool = True, 
 ) -> None | dict:
-    record = find(con, "SELECT dir_count, file_count, tree_dir_count, tree_file_count FROM dirlen WHERE id=?", id)
-    if record is None:
-        return None
-    return dict(zip(("dir_count", "file_count", "tree_dir_count", "tree_file_count"), record))
+    """获取某个目录里的文件数和目录数统计
+    """
+    sql = "SELECT dir_count, file_count, tree_dir_count, tree_file_count FROM dirlen WHERE id=?"
+    if is_alive:
+        sql += " AND is_alive"
+    return find(con, sql, id, row_factory="dict")
 
 
 def has_id(
@@ -71,7 +77,7 @@ def iter_existing_id(
     sql = "SELECT id FROM data WHERE id IN (%s)" % (",".join(map("%d".__mod__, ids)) or "NULL")
     if is_alive:
         sql += " AND is_alive"
-    return (id for id, in query(con, sql))
+    return query(con, sql, row_factory="one")
 
 
 def get_parent_id(
@@ -83,10 +89,7 @@ def get_parent_id(
     if id == 0:
         return 0
     sql = "SELECT parent_id FROM data WHERE id=?"
-    pid = find(con, sql, id, default)
-    if pid is None:
-        raise FileNotFoundError(ENOENT, id)
-    return pid
+    return find(con, sql, id, FileNotFoundError(ENOENT, id) if default is None else default)
 
 
 def iter_parent_id(
@@ -95,7 +98,7 @@ def iter_parent_id(
     /, 
 ) -> Iterator[int]:
     sql = "SELECT parent_id FROM data WHERE id IN (%s)" % (",".join(map("%d".__mod__, ids)) or "NULL")
-    return (id for id, in query(con, sql))
+    return query(con, sql, row_factory="one")
 
 
 def iter_id_to_parent_id(
@@ -103,7 +106,7 @@ def iter_id_to_parent_id(
     ids: Iterable[int], 
     /, 
     recursive: bool = False, 
-):
+) -> Iterator[tuple[int, int]]:
     s_ids = "(%s)" % (",".join(map(str, ids)) or "NULL")
     if recursive:
         sql = """\
@@ -149,14 +152,13 @@ def iter_id_to_path(
     else:
         patht = ("", *filter(None, path))
     if not parent_id and len(patht) == 1:
-        yield 0
-        return
+        return iter((0,))
     if len(patht) > 2:
         sql = "SELECT id FROM data WHERE parent_id=? AND name=? AND is_alive AND is_dir LIMIT 1"
         for name in patht[1:-1]:
             parent_id = find(con, sql, (parent_id, name), default=-1)
             if parent_id < 0:
-                return
+                return iter(())
     sql = "SELECT id FROM data WHERE parent_id=? AND name=? AND is_alive"
     if ensure_file is None:
         sql += " ORDER BY is_dir DESC"
@@ -164,8 +166,7 @@ def iter_id_to_path(
         sql += " AND NOT is_dir"
     else:
         sql += " AND is_dir LIMIT 1"
-    for id, in query(con, sql, (parent_id, patht[-1])):
-        yield id
+    return query(con, sql, (parent_id, patht[-1]), row_factory="one")
 
 
 def id_to_path(
@@ -378,32 +379,25 @@ def get_attr(
             "is_dir": 1, "type": 0, "ctime": 0, "mtime": 0, "is_collect": 0, 
             "is_alive": 1, "updated_at": datetime.fromtimestamp(0), 
         }
-    record = next(query(con, "SELECT * FROM data WHERE id=? LIMIT 1", id), None)
-    if record is None:
-        raise FileNotFoundError(ENOENT, id)
-    return dict(zip(FIELDS, record))
+    return find(
+        con, 
+        f"SELECT {','.join(FIELDS)} FROM data WHERE id=? LIMIT 1", 
+        id, 
+        FileNotFoundError(ENOENT, id), 
+        row_factory="dict", 
+    )
 
 
 def iter_children(
     con: Connection | Cursor, 
     parent_id: int | dict = 0, 
     /, 
-    fields: tuple[str, ...] = FIELDS, 
     ensure_file: None | bool = None, 
 ) -> Iterator[dict]:
     """获取某个目录之下的文件或目录的信息
 
     :param con: 数据库连接或游标
     :param parent_id: 父目录的 id
-    :param fields: 需要获取的字段，接受如下这些
-
-        .. code:: python
-
-            (
-                "id", "parent_id", "pickcode", "sha1", "name", "size", "is_dir", "type", 
-                "ctime", "mtime", "is_collect", "is_alive", "updated_at", 
-            )
-
     :param ensure_file: 是否仅输出文件
 
         - 如果为 True，仅输出文件
@@ -416,79 +410,141 @@ def iter_children(
         attr = get_attr(con, parent_id)
     else:
         attr = parent_id
-    if not fields:
-        fields = FIELDS
     if not attr["is_dir"]:
         raise NotADirectoryError(ENOTDIR, attr)
-    sql = "SELECT %s FROM data WHERE parent_id=? AND is_alive" % ",".join(fields)
-    if ensure_file:
-        sql += " AND NOT is_dir"
-    elif ensure_file is not None:
-        sql += " AND is_dir"
-    return (dict(zip(fields, record)) for record in query(con, sql, attr["id"]))
+    sql = f"SELECT {','.join(FIELDS)} FROM data WHERE parent_id=? AND is_alive"
+    if ensure_file is not None:
+        if ensure_file:
+            sql += " AND NOT is_dir"
+        else:
+            sql += " AND is_dir"
+    return query(con, sql, attr["id"], row_factory="dict")
 
 
 def iter_descendants(
     con: Connection | Cursor, 
     parent_id: int | dict = 0, 
     /, 
-    topdown: None | bool = True, 
+    min_depth: int = 1, 
     max_depth: int = -1, 
-    fields: tuple[str, ...] = FIELDS, 
     ensure_file: None | bool = None, 
+    use_relpath: None | bool = False, 
+    with_root: bool = False, 
+    topdown: None | bool = True, 
 ) -> Iterator[dict]:
     """遍历获取某个目录之下的所有文件或目录的信息
 
     :param con: 数据库连接或游标
     :param parent_id: 顶层目录的 id
+    :param min_depth: 最小深度
+    :param max_depth: 最大深度。如果小于 0，则无限深度
+    :param ensure_file: 是否仅输出文件
+
+        - 如果为 True，仅输出文件
+        - 如果为 False，仅输出目录
+        - 如果为 None，全部输出
+
+    :param use_relpath: 是否仅输出相对路径。如果为 False，则输出完整路径（从 / 开始）；如果为 None，则不输出 "ancestors", "path", "posixpath"
+    :param with_root: 仅当 `use_relpath=True` 时生效。如果为 True，则相对路径包含 `parent_id` 对应的节点
     :param topdown: 是否自顶向下深度优先遍历
 
         - 如果为 True，则自顶向下深度优先遍历
         - 如果为 False，则自底向上深度优先遍历
         - 如果为 None，则自顶向下宽度优先遍历
 
-    :param max_depth: 最大深度。如果小于 0，则无限深度
-    :param fields: 需要获取的字段，接受如下这些
+    :return: 迭代器，产生一组信息的字典，包含如下字段：
 
         .. code:: python
 
             (
-                "id", "parent_id", "pickcode", "sha1", "name", "size", "is_dir", "type", 
-                "ctime", "mtime", "is_collect", "is_alive", "updated_at", 
+                "id", "parent_id", "pickcode", "sha1", "name", "size", "is_dir", 
+                "type", "ctime", "mtime", "is_collect", "is_alive", "updated_at", 
+                "depth", "ancestors", "path", "posixpath", 
             )
-
-    :param ensure_file: 是否仅输出文件
-
-        - 如果为 True，仅输出文件
-        - 如果为 False，仅输出目录
-        - 如果为 None，全部输出
-
-    :return: 迭代器，产生一组信息的字典
     """
+    with_path = use_relpath is not None
     if isinstance(parent_id, int):
-        ancestors = get_ancestors(con, parent_id)
-        dir_ = "/".join(escape(a["name"]) for a in ancestors) + "/"
-        posixdir = "/".join(a["name"].replace("/", "|") for a in ancestors) + "/"
+        if 0 <= max_depth < min_depth:
+            return
+        depth = 1
+        if with_path:
+            if not parent_id:
+                ancestors: list[dict] = [{"id": 0, "parent_id": 0, "name": ""}]
+                dir_ = posixdir = "/"
+            elif use_relpath:
+                if with_root:
+                    attr = parent_id = get_attr(con, parent_id)
+                    name = attr["name"]
+                    ancestors = [{"id": attr["id"], "parent_id": attr["parent_id"], "name": name}]
+                    dir_ = escape(name) + "/"
+                    posixdir = name.replace("/", "|") + "/"
+                else:
+                    ancestors = []
+                    dir_ = posixdir = ""
+            else:
+                ancestors = get_ancestors(con, parent_id)
+                dir_ = "/".join(escape(a["name"]) for a in ancestors) + "/"
+                posixdir = "/".join(a["name"].replace("/", "|") for a in ancestors) + "/"
     else:
         attr = parent_id
-        ancestors = attr["ancestors"]
-        dir_ = attr["path"]
-        posixdir = attr["posixpath"]
+        depth = attr["depth"] + 1
+        if with_path:
+            ancestors = attr["ancestors"]
+            dir_ = attr["path"]
+            posixdir = attr["posixpath"]
+            if dir_ != "/":
+                dir_ += "/"
+                posixdir += "/"
     if topdown is None:
-        gen = bfs_gen((parent_id, max_depth, ancestors, dir_, posixdir))
-        send = gen.send
-        for parent_id, depth, ancestors, dir_, posixdir in gen:
-            depth -= depth > 0
-            for attr in iter_children(con, parent_id, fields=fields):
-                ancestors = attr["ancestors"] = [
+        if with_path:
+            gen = bfs_gen((parent_id, 0, ancestors, dir_, posixdir))
+        else:
+            gen = bfs_gen((parent_id, 0)) # type: ignore
+        send: Callable = gen.send
+        p: list
+        for parent_id, depth, *p in gen:
+            depth += 1
+            will_step_in = max_depth < 0 or depth < max_depth
+            will_yield = min_depth <= depth and (max_depth < 0 or depth <= max_depth)
+            if with_path:
+                ancestors, dir_, posixdir = p
+            for attr in iter_children(con, parent_id, False if ensure_file is False else None):
+                attr["depth"] = depth
+                is_dir = attr["is_dir"]
+                if with_path:
+                    attr["ancestors"] = [
+                        *ancestors, 
+                        {k: attr[k] for k in ("id", "parent_id", "name")}, 
+                    ]
+                    attr["path"] = dir_ + escape(attr["name"])
+                    attr["posixpath"] = posixdir + attr["name"].replace("/", "|")
+                if is_dir and will_step_in:
+                    if with_path:
+                        send((attr, depth, attr["ancestors"], attr["path"] + "/", attr["posixpath"] + "/"))
+                    else:
+                        send((attr, depth))
+                if will_yield:
+                    if ensure_file is None:
+                        yield attr
+                    elif is_dir:
+                        if not ensure_file:
+                            yield attr
+                    elif ensure_file:
+                        yield attr
+    else:
+        will_step_in = max_depth < 0 or depth < max_depth
+        will_yield = min_depth <= depth and (max_depth < 0 or depth <= max_depth)
+        for attr in iter_children(con, parent_id, False if ensure_file is False else None):
+            is_dir = attr["is_dir"]
+            attr["depth"] = depth
+            if with_path:
+                attr["ancestors"] = [
                     *ancestors, 
                     {k: attr[k] for k in ("id", "parent_id", "name")}, 
                 ]
-                dir_ = attr["path"] = dir_ + escape(attr["name"])
-                posixdir = attr["posixpath"] = posixdir + attr["name"].replace("/", "|")
-                is_dir = attr["is_dir"]
-                if is_dir and depth:
-                    send((attr, depth, ancestors, dir_, posixdir)) # type: ignore
+                attr["path"] = dir_ + escape(attr["name"])
+                attr["posixpath"] = posixdir + attr["name"].replace("/", "|")
+            if will_yield and topdown:
                 if ensure_file is None:
                     yield attr
                 elif is_dir:
@@ -496,33 +552,18 @@ def iter_descendants(
                         yield attr
                 elif ensure_file:
                     yield attr
-    else:
-        max_depth -= max_depth > 0
-        for attr in iter_children(con, parent_id, fields=fields):
-            is_dir = attr["is_dir"]
-            attr["ancestors"] = [
-                *ancestors, 
-                {k: attr[k] for k in ("id", "parent_id", "name")}, 
-            ]
-            attr["path"] = dir_ + escape(attr["name"])
-            attr["posixpath"] = posixdir + attr["name"].replace("/", "|")
-            if topdown:
-                if ensure_file is None:
-                    yield attr
-                elif is_dir:
-                    if not ensure_file:
-                        yield attr
-                elif ensure_file:
-                    yield attr
-            if is_dir and max_depth:
+            if is_dir and will_step_in:
                 yield from iter_descendants(
                     con, 
                     attr, 
-                    topdown=topdown, 
+                    min_depth=min_depth, 
                     max_depth=max_depth, 
                     ensure_file=ensure_file, 
+                    use_relpath=use_relpath, 
+                    with_root=with_root, 
+                    topdown=topdown, 
                 )
-            if not topdown:
+            if will_yield and not topdown:
                 if ensure_file is None:
                     yield attr
                 elif is_dir:
@@ -533,75 +574,67 @@ def iter_descendants(
 
 
 @overload
-def iter_descendants_fast(
+def iter_descendants_bfs(
     con: Connection | Cursor, 
     parent_id: int = 0, 
     /, 
-    *, 
-    fields: Literal[False], 
+    min_depth: int = 1, 
     max_depth: int = -1, 
     ensure_file: None | bool = None, 
-    where: str = "", 
-    orderby: str = "", 
-) -> Iterator[int]:
+    use_relpath: bool = False, 
+    with_root: bool = False, 
+    *, 
+    fields: str, 
+) -> Iterator[Any]:
     ...
 @overload
-def iter_descendants_fast(
+def iter_descendants_bfs(
     con: Connection | Cursor, 
     parent_id: int = 0, 
     /, 
-    *, 
-    fields: Literal[True] | tuple[str, ...] = True, 
+    min_depth: int = 1, 
     max_depth: int = -1, 
     ensure_file: None | bool = None, 
+    use_relpath: bool = False, 
+    with_root: bool = False, 
+    *, 
+    fields: tuple[str, ...] = EXTENDED_FIELDS, 
+    to_dict: Literal[False], 
+) -> Iterator[tuple[Any, ...]]:
+    ...
+@overload
+def iter_descendants_bfs(
+    con: Connection | Cursor, 
+    parent_id: int = 0, 
+    /, 
+    min_depth: int = 1, 
+    max_depth: int = -1, 
+    ensure_file: None | bool = None, 
+    use_relpath: bool = False, 
+    with_root: bool = False, 
+    *, 
+    fields: tuple[str, ...] = EXTENDED_FIELDS, 
     to_dict: Literal[True] = True, 
-    where: str = "", 
-    orderby: str = "", 
 ) -> Iterator[dict[str, Any]]:
     ...
-@overload
-def iter_descendants_fast(
+def iter_descendants_bfs(
     con: Connection | Cursor, 
     parent_id: int = 0, 
     /, 
-    *, 
-    fields: Literal[True] | tuple[str, ...] = True, 
+    min_depth: int = 1, 
     max_depth: int = -1, 
     ensure_file: None | bool = None, 
-    to_dict: Literal[False], 
-    where: str = "", 
-    orderby: str = "", 
-) -> Iterator:
-    ...
-def iter_descendants_fast(
-    con: Connection | Cursor, 
-    parent_id: int = 0, 
-    /, 
+    use_relpath: bool = False, 
+    with_root: bool = False, 
     *, 
-    fields: bool | tuple[str, ...] = True, 
-    max_depth: int = -1, 
-    ensure_file: None | bool = None, 
+    fields: str | tuple[str, ...] = EXTENDED_FIELDS, 
     to_dict: bool = True, 
-    where: str = "", 
-    orderby: str = "", 
 ) -> Iterator:
-    """获取某个目录之下的所有目录节点的 id 或者信息字典
+    """获取某个目录之下的所有目录节点的 id 或者信息字典（宽度优先遍历）
 
     :param con: 数据库连接或游标
     :param parent_id: 顶层目录的 id
-    :param fields: 需要获取的字段，接受如下这些，其中无论是否提供，"id" 必会被包含：
-
-        .. code:: python
-
-            (
-                "id", "parent_id", "pickcode", "sha1", "name", "size", "is_dir", "type", 
-                "ctime", "mtime", "is_collect", "is_alive", "updated_at", "path", "posixpath", 
-            )
-
-        - 如果为 False，则只拉取 id
-        - 如果为 True，则拉取上面提到的所有字段
-        - 如果为 tuple，则是指定了所要拉取的一组字段，无论是否提供，必会包含 "id"，且在最前面，且会把 "path" 和 "posixpath" 放在最后（如果有的话）
-
+    :param min_depth: 最小深度
     :param max_depth: 最大深度。如果小于 0，则无限深度
     :param ensure_file: 是否仅输出文件
 
@@ -609,115 +642,160 @@ def iter_descendants_fast(
         - 如果为 False，仅输出目录
         - 如果为 None，全部输出
 
-    :param to_dict: 是否产生字典，如果为 False，则直接迭代返回游标的数据
-    :param where: 一些 WHERE 查询条件，直接拼接到查询语句最后
-    :param orderby: 一些 ORDER BY 排序条件，直接拼接到查询语句最后
+    :param use_relpath: 仅输出相对路径，否则输出完整路径（从 / 开始）
+    :param with_root: 仅当 `use_relpath=True` 时生效。如果为 True，则相对路径包含 `parent_id` 对应的节点
+    :param fields: 需要获取的字段，接受如下这些：
 
-    :return: 迭代器，产生一组 id
+        .. code:: python
+
+            (
+                "id", "parent_id", "pickcode", "sha1", "name", "size", "is_dir", 
+                "type", "ctime", "mtime", "is_collect", "is_alive", "updated_at", 
+                "depth", "ancestors", "path", "posixpath", 
+            )
+
+        - 如果为 str，则获取指定的字段的值
+        - 如果为 tuple，则拉取这一组字段的值（但会过滤掉不可用的）
+
+    :param to_dict: 是否产生字典，如果为 True 且 fields 不为 str，则产生字典
+
+    :return: 迭代器，产生一组数据
     """
-    if fields is False:
-        if 0 <= max_depth <= 1:
-            sql = "SELECT id FROM data WHERE parent_id=:parent_id AND is_alive"
-        else:
-            if max_depth < 0:
-                args = ("", "", "")
-            else:
-                args = (
-                    ", 1 AS depth", 
-                    ", t.depth + 1", 
-                    " AND depth < :max_depth", 
-                )
-            if ensure_file is not None:
-                args = (
-                    args[0] + ", is_dir", 
-                    args[1] + ", data.is_dir", 
-                    args[2], 
-                )
-            sql = """\
-WITH t AS (
-    SELECT id%s FROM data WHERE parent_id=:parent_id AND is_alive
-    UNION ALL
-    SELECT data.id%s FROM t JOIN data ON(t.id = data.parent_id) WHERE data.is_alive%s
-)
-SELECT id FROM t AS data WHERE TRUE""" % args
-        if ensure_file:
-            sql += " AND NOT is_dir"
-        elif ensure_file is not None:
-            sql += " AND is_dir"
-        if where:
-            sql += f" AND ({where})"
-        if orderby:
-            sql += f" ORDER BY {orderby}"
-        return (id for id, in query(con, sql, locals()))
+    one_value = False
+    parse: None | Callable = None
+    if isinstance(fields, str):
+        one_value = True
+        field = fields
+        if field not in EXTENDED_FIELDS:
+            raise ValueError(f"invalid field {field!r}, must be in {EXTENDED_FIELDS!r}")
+        with_id = "id" == field
+        with_depth = max_depth > 1 or "depth" == field
+        with_ancestors = "ancestors" == field
+        with_path = "path" == field
+        with_posixpath = "posixpath" == field
+        fields = field,
     else:
-        if fields is True:
-            fields = EXTENDED_FIELDS
-        fields = cast(tuple[str, ...], fields)
+        fields = tuple(filter(set(EXTENDED_FIELDS).__contains__, fields))
+        with_id = "id" in fields
+        with_depth = max_depth > 1 or min_depth > 1 or "depth" in fields
+        with_ancestors = "ancestors" in fields
         with_path = "path" in fields
         with_posixpath = "posixpath" in fields
-        if with_path:
-            try:
-                (con.connection if isinstance(con, Cursor) else con).create_function(
-                    "escape_name", 1, escape, deterministic=True)
-            except OperationalError:
-                pass
-        if with_path or with_posixpath:
-            if parent_id:
-                ancestors = get_ancestors(con, parent_id)
-                if with_path:
-                    path = "/".join(escape(a["name"]) for a in ancestors)
-                if with_posixpath:
-                    posixpath = "/".join(a["name"].replace("/", "|") for a in ancestors)
+    select_fields1 = ["id"]
+    select_fields1.extend(set(fields) & set(FIELDS[1:]))
+    select_fields2 = ["data." + f for f in select_fields1]
+    where1, where2 = "", ""
+    if with_depth:
+        select_fields1.append("1 AS depth")
+        select_fields2.append("t.depth + 1")
+    if with_path or with_posixpath or with_ancestors:
+        if not parent_id:
+            ancestors: list[dict] = [{"id": 0, "parent_id": 0, "name": ""}]
+            path = posixpath = "/"
+        elif use_relpath:
+            if with_root:
+                attr = get_attr(con, parent_id)
+                name = attr["name"]
+                ancestors = [{"id": attr["id"], "parent_id": attr["parent_id"], "name": name}]
+                path = escape(name) + "/"
+                posixpath = name.replace("/", "|") + "/"
             else:
+                ancestors = []
                 path = posixpath = ""
-        predicate: Callable[[str], bool] = frozenset(FIELDS[1:]).__contains__
-        fields = ("id", *filter(predicate, fields))
-        select_fields_1 = ", ".join(fields)
-        select_fields_2 = "data." + ", data.".join(fields)
-        if with_path:
-            select_fields_1 += ", :path || '/' || escape_name(name) AS path"
-            select_fields_2 += ", t.path || '/' || escape_name(data.name)"
-            fields += "path",
-        if with_posixpath:
-            select_fields_1 += ", :posixpath || '/' || REPLACE(name, '/', '|') AS posixpath"
-            select_fields_2 += ", t.posixpath || '/' || REPLACE(data.name, '/', '|')"
-            fields += "posixpath",
-        if 0 <= max_depth <= 1:
-            sql = f"SELECT {select_fields_1} FROM data WHERE parent_id=:parent_id AND is_alive"
         else:
-            if max_depth < 0:
-                args = ("", "", "")
+            ancestors = get_ancestors(con, parent_id)
+            if with_path:
+                path = "/".join(escape(a["name"]) for a in ancestors) + "/"
+            if with_posixpath:
+                posixpath = "/".join(a["name"].replace("/", "|") for a in ancestors) + "/"
+        if with_ancestors:
+            if ancestors:
+                parse_ancestors = lambda val: [*ancestors, *loads("[%s]" % val)]
             else:
-                args = (
-                    ", 1 AS depth", 
-                    ", t.depth + 1", 
-                    " AND depth < :max_depth", 
-                )
-            if ensure_file is not None:
-                args = (
-                    args[0] + ", is_dir", 
-                    args[1] + ", data.is_dir", 
-                    args[2], 
-                )
-            sql = f"""\
+                parse_ancestors = lambda val: loads("[%s]" % val)
+            parse = parse_ancestors
+            select_fields1.append("json_object('id', id, 'parent_id', parent_id, 'name', name) AS ancestors")
+            select_fields2.append("concat(t.ancestors, ',', json_object('id', data.id, 'parent_id', data.parent_id, 'name', data.name))")
+        if with_path:
+            def parse_path(val: str, /) -> str:
+                return path + val
+            parse = parse_path
+            if isinstance(con, Cursor):
+                conn = con.connection
+            else:
+                conn = con
+            conn.create_function("escape_name", 1, escape, deterministic=True)
+            select_fields1.append("escape_name(name) AS path")
+            select_fields2.append("concat(t.path, '/', escape_name(data.name))")
+        if with_posixpath:
+            def parse_posixpath(val: str, /) -> str:
+                return posixpath + val
+            parse = parse_posixpath
+            select_fields1.append("replace(name, '/', '|') AS posixpath")
+            select_fields2.append("concat(t.posixpath, '/', replace(data.name, '/', '|'))")
+    if min_depth <= 1 and max_depth in (0, 1) or 0 <= max_depth < min_depth:
+        if 0 <= max_depth < min_depth:
+            where1 = " AND FALSE"
+        elif ensure_file:
+            where1 = " AND NOT is_dir"
+        elif ensure_file is False:
+            where1 = " AND is_dir"
+        sql = f"""\
 WITH t AS (
-    SELECT {select_fields_1}%s FROM data WHERE parent_id=:parent_id AND is_alive
+    SELECT {",".join(select_fields1)} FROM data WHERE parent_id={parent_id:d} AND is_alive{where1}
+) SELECT {",".join(fields)} FROM t"""
+    else:
+        if max_depth > 1:
+            where2 = f" AND depth < {max_depth:d}"
+        if ensure_file:
+            if "is_dir" not in fields:
+                select_fields1.append("is_dir")
+                select_fields2.append("data.is_dir")
+        elif ensure_file is False:
+            where1 += " AND is_dir"
+            where2 += " AND data.is_dir"
+        sql = f"""\
+WITH t AS (
+    SELECT {",".join(select_fields1)} FROM data WHERE parent_id={parent_id:d} AND is_alive{where1}
     UNION ALL
-    SELECT {select_fields_2}%s FROM t JOIN data ON(t.id = data.parent_id) WHERE data.is_alive%s
-)
-SELECT {",".join(fields)} FROM t AS data WHERE TRUE""" % args
+    SELECT {",".join(select_fields2)} FROM t JOIN data ON(t.id = data.parent_id) WHERE data.is_alive{where2}
+) SELECT {",".join(fields)} FROM t WHERE True"""
         if ensure_file:
             sql += " AND NOT is_dir"
-        elif ensure_file is not None:
-            sql += " AND is_dir"
-        if where:
-            sql += f" AND ({where})"
-        if orderby:
-            sql += f" ORDER BY {orderby}"
-        cur = query(con, sql, locals())
-        if to_dict:
-            return (dict(zip(fields, record)) for record in cur)
-        return cur
+        if min_depth > 1:
+            sql += f" AND depth >= {min_depth:d}"
+    if one_value:
+        if parse is None:
+            row_factory = lambda _, r: r[0]
+        else:
+            row_factory = lambda _, r: parse(r[0])
+    elif to_dict:
+        def row_factory(_, r):
+            d = dict(zip(fields, r))
+            if with_ancestors:
+                d["ancestors"] = parse_ancestors(d["ancestors"])
+            if with_path:
+                d["path"] = parse_path(d["path"])
+            if with_posixpath:
+                d["posixpath"] = parse_posixpath(d["posixpath"])
+            return d
+    else:
+        with_route = with_ancestors or with_path or with_posixpath
+        def parse(f, v):
+            match f:
+                case "ancestors":
+                    return parse_ancestors(v)
+                case "path":
+                    return parse_path(v)
+                case "posixpath":
+                    return parse_posixpath(v)
+                case _:
+                    return v
+        def row_factory(_, r):
+            if with_route:
+                return tuple(parse(f, v) for f, v in zip(fields, r))
+            return r
+    return query(con, sql, row_factory=row_factory)
 
 
 def iter_files_with_path_url(
@@ -734,16 +812,89 @@ def iter_files_with_path_url(
 
     :return: 迭代器，返回每个文件的 路径 和 下载链接 的 2 元组
     """
+    from encode_uri import encode_uri_component_loose
     if isinstance(parent_id, str):
         parent_id = get_id(con, path=parent_id)
     code = compile('f"%s/{quote(name, '"''"')}?{id=}&{pickcode=!s}&{sha1=!s}&{size=}&file=true"' % base_url.translate({ord(c): c*2 for c in "{}"}), "-", "eval")
-    for attr in iter_descendants_fast(
+    for attr in iter_descendants_bfs(
         con, 
         parent_id, 
         fields=("id", "sha1", "pickcode", "size", "name", "posixpath"), 
         ensure_file=True, 
     ):
-        yield attr["posixpath"], eval(code, None, attr)
+        yield attr["posixpath"], eval(code, {"quote": encode_uri_component_loose}, attr)
+
+
+def iter_dup_files(
+    con: Connection | Cursor, 
+    /, 
+) -> Iterator[dict]:
+    """罗列所有重复文件
+
+    :param con: 数据库连接或游标
+
+    :return: 迭代器，一组文件的信息
+    """
+    sql = f"""\
+WITH stats AS (
+    SELECT
+        COUNT(1) OVER w AS total, 
+        ROW_NUMBER() OVER w AS nth, 
+        {",".join(FIELDS)}
+    FROM data
+    WHERE NOT is_dir AND is_alive
+    WINDOW w AS (PARTITION BY sha1, size)
+)
+SELECT * FROM stats WHERE total > 1"""
+    return query(con, sql, row_factory="dict")
+
+
+def iter_dangling_parent_ids(
+    con: Connection | Cursor, 
+    /, 
+) -> Iterator[int]:
+    """罗列所有悬空的 parent_id
+
+    .. note::
+        悬空的 parent_id，即所有的 parent_id 中，，不为 0 且不在 `data` 表中的部分
+
+    :param con: 数据库连接或游标
+
+    :return: 迭代器，一组目录的 id
+    """
+    sql = """\
+SELECT
+    DISTINCT d1.parent_id
+FROM
+    data AS d1 LEFT JOIN data AS d2 ON (d1.parent_id = d2.id)
+WHERE
+    d1.parent_id AND d2.id IS NULL"""
+    return query(con, sql, row_factory="one")
+
+
+def iter_dangling_ids(
+    con: Connection | Cursor, 
+    /, 
+) -> Iterator[int]:
+    """罗列所有悬空的文件或目录的 id
+
+    .. note::
+        悬空的 id，即祖先节点中，存在一个节点，它的 parent_id 是悬空的
+
+    :param con: 数据库连接或游标
+
+    :return: 迭代器，一组目录的 id
+    """
+    sql = """\
+WITH dangling_ids(id) AS (
+    SELECT d1.id
+    FROM data AS d1 LEFT JOIN data AS d2 ON (d1.parent_id = d2.id)
+    WHERE d1.parent_id AND d2.id IS NULL
+    UNION ALL
+    SELECT data.id FROM dangling_ids JOIN data ON (dangling_ids.id = data.parent_id)
+)
+SELECT id FROM dangling_ids"""
+    return query(con, sql, row_factory="one")
 
 
 def select_na_ids(
@@ -753,13 +904,13 @@ def select_na_ids(
     """找出所有的失效节点和悬空节点的 id
 
     .. note::
-        悬空节点，就是此节点有一个祖先节点的 id，不为 0 且不在 `data` 表中
+        悬空节点，就是此节点有一个祖先节点的 parant_id，不为 0 且不在 `data` 表中
 
     :param con: 数据库连接或游标
 
     :return: 一组悬空节点的 id 的集合
     """
-    ok_ids: set[int] = set(query(con, "SELECT id FROM data WHERE NOT is_alive"))
+    ok_ids: set[int] = set(query(con, "SELECT id FROM data WHERE NOT is_alive", row_factory="one"))
     na_ids: set[int] = set()
     d = dict(query(con, "SELECT id, parent_id FROM data WHERE is_alive"))
     temp: list[int] = []
@@ -792,27 +943,21 @@ def select_mtime_groups(
     parent_id: int = 0, 
     /, 
     tree: bool = False, 
-) -> tuple[int, list[tuple[int, set[int]]]]:
+) -> list[tuple[int, set[int]]]:
     """获取某个目录之下的节点（不含此节点本身），按 mtime 进行分组，相同 mtime 的 id 归入同一组
 
     :param con: 数据库连接或游标
     :param parent_id: 父目录的 id
     :param tree: 是否拉取目录树，如果为 True，则拉取全部后代的文件节点（不含目录节点），如果为 False，则只拉取子节点（含目录节点）
 
-    :return: 字典，表示相同 mtime 的 id 的集合，所以 key 是 mtime，value 是一组 id 的集合
+    :return: 元组的列表（逆序排列），每个元组第 1 个元素是 mtime，第 2 个元素是相同 mtime 的 id 的集合
     """
     if tree:
-        it = iter_descendants_fast(con, parent_id, fields=("id", "mtime"), ensure_file=True, to_dict=False)
+        it = iter_descendants_bfs(con, parent_id, fields=("mtime", "id"), ensure_file=True, to_dict=False)
     else:
-        it = iter_descendants_fast(con, parent_id, fields=("id", "mtime"), max_depth=1, to_dict=False)
-    d: dict[int, set[int]] = {}
-    n = 0
-    for n, (id, mtime) in enumerate(it, 1):
-        try:
-            d[mtime].add(id)
-        except KeyError:
-            d[mtime] = {id}
-    return n, sorted(d.items(), reverse=True)
+        it = iter_descendants_bfs(con, parent_id, fields=("mtime", "id"), max_depth=1, to_dict=False)
+    d: dict[int, set[int]] = group_collect(it, factory=set)
+    return sorted(d.items(), reverse=True)
 
 
 def dump_to_alist(
@@ -843,7 +988,7 @@ WITH t AS (
         is_dir, 
         size, 
         id, 
-        CASE WHEN is_dir THEN :dirname || '/' || REPLACE(name, '/', '|') END AS dirname 
+        CASE WHEN is_dir THEN CONCAT(:dirname, '/', REPLACE(name, '/', '|')) END AS dirname 
     FROM data WHERE parent_id=:parent_id AND is_alive
     UNION ALL
     SELECT 
@@ -852,20 +997,63 @@ WITH t AS (
         data.is_dir, 
         data.size, 
         data.id, 
-        CASE WHEN data.is_dir THEN t.dirname || '/' || REPLACE(data.name, '/', '|') END AS dirname
+        CASE WHEN data.is_dir THEN CONCAT(t.dirname, '/', REPLACE(data.name, '/', '|')) END AS dirname
     FROM t JOIN data ON(t.id = data.parent_id) WHERE data.is_alive
 )
-SELECT * FROM t"""
+SELECT parent, name, is_dir, size FROM t"""
     dirname = "/" + dirname.strip("/")
     with transact(alist_db) as cur:
         if clean:
             cur.execute("DELETE FROM x_search_nodes WHERE parent=? OR parent LIKE ? || '/%';", (dirname, dirname))
         count = 0
-        it = (t[:4] for t in query(con, sql, locals()))
         executemany = cur.executemany
-        for items in batched(it, 10_000):
+        for items in batched(query(con, sql, locals()), 10_000):
             executemany("INSERT INTO x_search_nodes(parent, name, is_dir, size) VALUES (?, ?, ?, ?)", items)
             count += len(items)
         return count
 
-# TODO: 增加函数，用来导出到 efu (everything)、mlocatedb 等软件的索引数据库
+
+def dump_efu(
+    con: Connection | Cursor, 
+    /, 
+    efu_file: str | Path = "export.efu", 
+    parent_id: int | str = 0, 
+    dirname: str = "", 
+    use_relpath: bool = False, 
+) -> int:
+    """把 p115updatedb 导出的数据，导出为 efu 文件，可供 everything 软件使用
+
+    :param con: 数据库连接或游标
+    :param efu_file: 要导出的文件路径
+    :param parent_id: 在 p115updatedb 所导出数据库中的顶层目录 id 或路径
+    :param dirname: 给每个导出路径添加的目录前缀
+
+    :return: 总共导出的数量
+    """
+    def unix_to_filetime(unix_time: float, /) -> int:
+        return int(unix_time * 10 ** 7) + 11644473600 * 10 ** 7
+    if dirname:
+        dirname = normpath(dirname)
+        if not dirname.endswith("\\"):
+            dirname += "\\"
+    if isinstance(parent_id, str):
+        parent_id = get_id(con, path=parent_id)
+    n = 0
+    with open(efu_file, "w", newline="", encoding="utf-8") as file:
+        csvfile = writer(file)
+        writerow = csvfile.writerow
+        writerow(("Filename", "Size", "Date Modified", "Date Created", "Attributes"))
+        for n, (size, ctime, mtime, is_dir, path) in enumerate(iter_descendants_bfs(
+            con, 
+            parent_id, 
+            use_relpath=use_relpath, 
+            fields=("size", "ctime", "mtime", "is_dir", "posixpath"), 
+            to_dict=False, 
+        ), 1):
+            if use_relpath:
+                path = normpath(path)
+            else:
+                path = normpath(path[1:])
+            writerow((dirname + path, size, unix_to_filetime(mtime), unix_to_filetime(ctime), 16 if is_dir else 0))
+    return n
+
